/*--------------------------------------------------------------------------------
HELLO.C

Zdrojovy kod programu vypisuje textovy retazec Hello World! na rozhranie UART
(seriovu linku). Projekt vyuziva MCU ADuC 842 (zrychlene jadro 8051,
MCU vsak obsahuje viacero zaujimavych rozsireni - vid. kat. list ADuC842.pdf).

Projekt predpoklada taktovanie MCU pomocou 32.768 kHz krystalu. Interne je
taktovacia frevencia pomocou PLL nasobena na maximalnu taktovaciu frekvenciu.
(nastavi makro SET_FCLK). Pre tuto hodnotu je nastavena prenosova rychlost
rozhrania UART na 115200 baudov.

Program je mozne skompilovat a spustit pomocou rozhrania uVision
(s nainstalovanou podporou pre C51)

2017-09-19, M.D.
--------------------------------------------------------------------------------*/

#include <stdio.h>
#include <ADuC836.h>

#define SET_FCLK        (PLLCON = 0x00)         // Fclk = 16.777216 MHz


void uart_init( void ) {
// UART config (s vyuzitim specialneho casovaca T3, ktory
// vyrazne rozsiruje moznosti ADuC 83x)
    SET_FCLK;        // nastavenie 12.582192 MHz

//UART config, T3 clock, (pozri strana 60 v ADuC 836 manuale)
    T3CON = 0x83;    // Timer 3 baud rate 115200, 16.777216 MHz core freq.
    T3FD = 0x09;     // delicka pre 115200 baudov
    SCON = 0x52;
}

void main (void)
{
/*
Konfiguracia serioveho portu pre 115200 baudov a 32.768 kHz krystal.
*/
  uart_init();

/*
Vsimnite si, ze program nikdy nekonci (pretoze nie je vyuzity ziadny
operacny system, ktoremu by bolo mozne odovzdat riadenie).
Nekonecna slucka je jeden zo sposobov, kotre su pri vstavanych (embedded)
aplikaciach typicky vyuzivane.
*/
  while (1) {
    P1 ^= 0x01;                 /* Zmeni hodnotu na vyvode P1.0 pred vypisom */
    printf ("Hello World!\n");  /* Vypise text "Hello World!" na UART  */
  }
}
